/* <!-- copyright */
/*
 * aria2 - The high speed download utility
 *
 * Copyright (C) 2006 Tatsuhiro Tsujikawa
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
 */
/* copyright --> */
#include "SingleFileAllocationIterator.h"
#include "BinaryStream.h"
#include "Util.h"
#include "a2io.h"

#define BUFSIZE (256*1024)
#define ALIGNMENT 512

SingleFileAllocationIterator::SingleFileAllocationIterator(BinaryStream* stream, int64_t offset, int64_t totalLength):_stream(stream), _offset(offset), _totalLength(totalLength), _buffer(0)
{
  if(_offset%ALIGNMENT != 0) {
    _stream->disableDirectIO();
  }
}

SingleFileAllocationIterator::~SingleFileAllocationIterator()
{
  delete [] _buffer;
}

void SingleFileAllocationIterator::init()
{
#ifdef HAVE_POSIX_MEMALIGN
  _buffer = (unsigned char*)Util::allocateAlignedMemory(ALIGNMENT, BUFSIZE);
#else
  _buffer = new unsigned char[BUFSIZE];
#endif // HAVE_POSIX_MEMALIGN
  memset(_buffer, 0, BUFSIZE);
}

void SingleFileAllocationIterator::allocateChunk()
{
  _stream->writeData(_buffer, BUFSIZE, _offset);
  _offset += BUFSIZE;

  if(_totalLength < _offset) {
    _stream->truncate(_totalLength);
    _offset = _totalLength;
  }
}

bool SingleFileAllocationIterator::finished()
{
  return _offset >= _totalLength;
}
